#include <stdio.h>
#include <stdlib.h>

#include "metrinit.h"

#include "globalc.h"
#include "globmatc.h"
#include "fcsolver.h"
#include "gtopology.h"

#include "seqfilesm.h"
#include "seqfilest.h"

#include "iotools.h"
#include "xfile.h"
#include "outdriverc.h"
#include "stacktrace.h"
#include "libtrace.h"


void metr_init (int argc, const char *argv[])
{
  long i,n;
  XFILE *in,*inm,*intr;

  set_prgname(argv[0]);
  fprintf (stderr,"\n\n ******************************************************\n");
  
  //  docasne umisteni
  if (Bar2d==NULL)   Bar2d = new barel2d;
  if (Beam2d==NULL)   Beam2d = new beamel2d;
  if (Pelq==NULL)    Pelq = new planeelemlq;
  if (Peqq==NULL)    Peqq = new planeelemqq;
  if (Asymlq==NULL)  Asymlq = new axisymlq;
  if (Lhex==NULL)  Lhex = new linhex;
  
  if (Ltet==NULL)  Ltet = new lintet;
  if (Bar3d==NULL)   Bar3d = new barel3d;

  //  problem description of all coupled problem
  Cp = new probdescc;
  
  if (process_argsc(argc, argv)){
    delete Cp;
    abort();
  }
  
  //  problem description of mechanical part of problem
  Mp = new probdesc;
  //  problem description of thermo-hydro part of problem
  Tp = new probdesct;
  //  topology of mechanical part of problem
  Gtm = new gtopology;
  //  topology of thermo-hydro part of problem
  Gtt = new gtopology;
  //  auxiliary united topology (mechanical + thermo-hydro)
  Gtu = new gtopology;
  //  additional topological data about mechanical part of problem
  Mt = new mechtop;
  //  additional topological data about thermo-hydro part of problem
  Tt = new transtop;
  //  additional topological data about thermo-hydro part of problem
  Ct = new couptop;
  //  materials of mechanical part of problem
  Mm = new mechmat;
  //  materials of thermo-hydro part of problem
  Tm = new transmat;
  //  materials of thermo-hydro part of problem
  Cmu = new coupmatu;
  //  materials of thermo-hydro part of problem
  Cml = new coupmatl;
  //  cross-section in mechanical part of problem
  Mc = new mechcrsec;
  //  cross-section in thermo-hydro part of problem
  Tc = new transcrsec;
  //  boundary conditions and load cases in coupled problem
  Cb = new coupbclc;
  //  boundary conditions and load cases in mechanical part of problem
  Mb = new mechbclc;
  //  boundary conditions and load cases in thermo-hydro part of problem
  Tb = new transbclc;
  //outdriver
  Outdm = new outdriverm;
  Outdt = new outdrivert;
  Outdc = new outdriverc;
  
  in = xfopen(argv[1],"r");
  if (in==NULL){
    print_err("input file cannot be opened.", __FILE__, __LINE__, __func__);
    abort ();
  }
  in->warning    = 1;
  in->ignorecase = 1;
  if ((Cp->kwdsw == nokwd) || (Cp->kwdsw == kwd_outd))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;

  //  problem description of all coupled problem
  Cp->read (in);
  
  //  input file for mechanical part
  inm = xfopen(Cp->minfile,"r");
  if (inm==NULL){
    print_err ("input file for mechanical part cannot be opened.", __FILE__, __LINE__, __func__);
    abort ();
  }

  //  input file for thermo-hydro part
  intr = xfopen(Cp->tinfile,"r");
  if (intr==NULL){
    print_err ("input file for thermo-hydro part cannot be opened.",__FILE__, __LINE__, __func__);
    abort ();
  }
  
  inm->warning    = 1;
  inm->ignorecase = 1;
  if ((Mp->kwdsw == nokwd) || (Mp->kwdsw == kwd_outd))
    inm->kwdmode = ignore;
  else
    inm->kwdmode = sequent_mode;

  intr->warning    = 1;
  intr->ignorecase = 1;
  if ((Tp->kwdsw == nokwd) || (Tp->kwdsw == kwd_outd))
    intr->kwdmode = ignore;
  else
    intr->kwdmode = sequent_mode;

  //  basic problem data
  Mp->read (inm);
  Tp->read (intr);
    
  Outc = fopen ("metr.log","wt");
  Out  = fopen ("mefel.log","wt");
  Outt = fopen ("trfel.log","wt");
  
  if (Mp->kwdsw == kwd_full)
    inm->kwdmode = sequent_mode;
  else
    inm->kwdmode = ignore;
  
  if (Tp->kwdsw == kwd_full)
    intr->kwdmode = sequent_mode;
  else
    intr->kwdmode = ignore;
  
  if (Cp->kwdsw == kwd_full)
    in->kwdmode = sequent_mode;
  else
    in->kwdmode = ignore;
  
  //  input of node and element data
  Mt->read (inm);
  Tt->read (intr);
  Ct->read (in);
  
  
  // mefel mesh control TKr
  /* if (Mp->tprob == growing_mech_structure){
     if (Mespr==1){
     merr=Mt->mesh_check();
     if (merr > 0){
     fprintf (stdout,"MEFEL Mesh error = %ld\n\n\n",merr);
     exit(0);
     }
     }
     }
  */
    
  // trfel mesh control
  /*if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    if (Mesprt==1){
    merr=Tt->mesh_check();
    if (merr > 0){
    fprintf (stdout,"TRFEL Mesh error = %ld\n\n\n",merr);
    exit(0);
    }
    }
    }
  */

  // metr mesh control
  /*
  if ((Cp->tprob==growing_par_coupl_mech_trans) || (Cp->tprob==par_coupl_mech_trans)){
    if (Mesprc==1){
      merr=Ct->mesh_check();
      if (merr > 0){
        fprintf (stdout,"METR Mesh error = %ld\n\n\n",merr);
        exit(0);
      }
    }
  }
  */

  if (Cp->tprob==fully_coupled_mech_trans){
    //  unification of two topologies
    Gtu->comptop (Gtm,Gtt);
    
    //  allocation and initiation of arrays lnso and leso
    //  lnso - list of nodes switched on
    //  leso - list of elements switched on
    //  11/05/2010 TKr
    Gtu->lneso_init ();
  }
  
  //  generation of code numbers in mechanical part
  Ndofm = Gtm->codenum_generation (Out);//Ndofm=Gtm->gencodnum ();
  if (Mesprc==1)
    fprintf (stdout,"\n number of degrees of freedom in mechanics    %ld",Ndofm);
  //Mt->store_code_num_elem(); // enable combination of plane elements, springs and bar elems with beams//tady????!!

  //  generation of code numbers in transport part
  Ndoft = Gtt->codenum_generation (Outt);//Ndoft=Gtt->gencodnum ();
  if (Mesprc==1)
    fprintf (stdout,"\n number of degrees of freedom in transports    %ld",Ndoft);
  
  if (Cp->tprob==fully_coupled_mech_trans){
    //  generation of code numbers in both parts
    Ndofc = Gtu->codenum_generation (Outc);//Ndofc=Gtu->gencodnum ();
    //  distribution of code numbers to local topologies
    Gtu->cndistr (Gtm,Gtt);
    
    Ndofm=Ndofc;
    Ndoft=Ndofc;
    
    if (Mesprc==1)
      fprintf (stdout,"\n total number of degrees of freedom    %ld",Ndofc);
  }

  //read trfel function
  if (Tp->tprob == growing_np_problem || Tp->tprob == growing_np_problem_nonlin){
    Gtt->read_gf (intr);
    Gtt->alloc_growstr();
  }

  //  input of material characteristics
  Mm->read (inm);
  Tm->read (intr);
  
  //int. points allocation and initiation
  Tm->intpointalloc ();
  Tm->intpointinit ();
  Tm->assemble_dof_nameord(Tp->dofname, Tp->ntm, Tp->var_dofid, Tp->tnkv);

  //  allocation of auxiliary arrays on integration points
  Mm->init_ip_1 ();

  if (Cp->tprob==fully_coupled_mech_trans){
    //Cmu->read (in->file);
    //Cml->read (in->file);
    Cmu->read (in);
    Cml->read (in);
  }

  //  input of cross section characteristics
  Mc->read (inm);
  Tc->read (intr);
  
  //  input of loading
  Mb->read (inm);
  Tb->read (intr);

  //read mefel function
  if (Mp->tprob == growing_mech_structure){
    Gtm->read_gf (inm);
    Gtm->alloc_growstr();
    Mt->alloc_growstr ();
  }
 
  //  prescribed values on elements investigation
  //Tt->elemprescvalue();
  //  searching for elements with nodes where source is defined
  Tb->elemsource ();
  
  // allocation of non-mechanical and non-transport arrays
  // definition of their ordering
  n = Mm->search_reqnmq(Mm->nmqo);
  if (n){
    for (i=0; i<n; i++)
      Mm->nmqid[Mm->nmqo[i]-1] = i;
    
    Mm->alloc_nonmechq(n);
  }
  
  n = Tm->search_reqntq(Tm->ntqo);
  if (n){
    for (i=0; i<n; i++)
      Tm->ntqid[Tm->ntqo[i]-1] = i;
    
    Tm->alloc_nontransq(n);
  }

  // create mapping between MEFEL and TRFEL integration points  
  TM_ip = new long[Tm->tnip];
  MT_ip = new long[Mm->tnip];
  mefel_trfel_ip_mapping_old(TM_ip, MT_ip);

  if (Cp->tprob==fully_coupled_mech_trans){  
    if (Mp->pore_press)   // effective stress concept was defined in the MEFEL
      Mp->pore_press = 3; // indicator of fully coupled approach

    //  left and right hand sides
    Lsrsc = new lhsrhsc;
    Lsrsc->alloc ();
    //Lsrsc->initcond (inc);
    
    Lsrs = new lhsrhs;
    Lsrs->lhsi=Lsrsc->lhsi; //initial conditions
    Lsrs->initcond ();
    
    Lsrst = new lhsrhst;     
    Lsrst->lhsi=Lsrsc->lhsi; //initial conditions
    Lsrst->initcond (intr);    
  }
  
  
  if ((Cp->tprob==par_coupl_mech_trans) || (Cp->tprob==growing_par_coupl_mech_trans)){
    if (Mp->pore_press)   // effective stress concept was defined in the MEFEL
      Mp->pore_press = 2; // indicator of partially coupled approach

    //  object for left and right hand side of MEFEL problem
    Lsrs = new lhsrhs;
    Lsrs->alloc ();
    Lsrs->initcond ();
    
    //  object for left and right hand side of TRFEL problem
    Lsrst = new lhsrhst;
    Lsrst->alloc ();
    Lsrst->initcond (intr);
  }
  
  Mb->alloc_sumcomp ();  // arrays for resultant force components
  if (Mm->csol)
    Gtm->comp_domain_sizes (); // maximum domain sizes used consolidation model

  // input of output description
  //metr
  if ((Cp->kwdsw == nokwd) || (Cp->kwdsw == kwd_pd))
    in->kwdmode = ignore;
  else
    in->kwdmode = sequent_mode;
    
  Outdc->read(in);
   
  //mefel
  if ((Mp->kwdsw == nokwd) || (Mp->kwdsw == kwd_pd))
    inm->kwdmode = ignore;
  else
    inm->kwdmode = sequent_mode;
  
  Outdm->read(inm);
  
  //trfel
  if ((Tp->kwdsw == nokwd) || (Tp->kwdsw == kwd_pd))
    intr->kwdmode = ignore;
  else
    intr->kwdmode = sequent_mode;
    
  Outdt->read(intr);

  fclose (Outc);
  fclose (Outt);
  fclose (Out);
  
  char fname[1020];
  if (*Outdc->outfn)
    filename_decomposition(Outdc->outfn,Cp->path,Cp->filename,Cp->suffix);
  else
    filename_decomposition(argv[1],Cp->path,Cp->filename,Cp->suffix);
  sprintf(fname, "%s%s.log", Cp->path, Cp->filename);
  Outc = fopen(fname,"wt");

  if (*Outdt->outfn)
    filename_decomposition(Outdt->outfn,Tp->path,Tp->filename,Tp->suffix);
  else
    filename_decomposition(Cp->tinfile,Tp->path,Tp->filename,Tp->suffix);
  if ((strlen(Tp->path) == 0) && (strlen(Tp->filename) == 0))
  {
    sprintf(fname, "trfel.log");
    Outt = fopen(fname,"at");
  }
  else
  {
    sprintf(fname, "%s%s.log", Tp->path, Tp->filename);
    Outt = fopen(fname,"wt");
  }
  
  if (*Outdm->outfn)
    filename_decomposition(Outdm->outfn,Mp->path,Mp->filename,Mp->suffix);
  else
    filename_decomposition(Cp->minfile,Mp->path,Mp->filename,Mp->suffix);
  if ((strlen(Mp->path) == 0) && (strlen(Mp->filename) == 0))
  {
    sprintf(fname, "mefel.log");
    Out = fopen(fname,"at");
  }
  else
  {
    sprintf(fname, "%s%s.log", Mp->path, Mp->filename);
    Out = fopen(fname,"wt");
  }
  fprintf(stdout, "\n Log file name : '%s'", fname);
  
  //Mm->stra.init(strain);
  //Mm->stre.init(stress);

  //  allocation of auxiliary arrays on integration points
  Mm->init_ip_2 ();

  //  nodes allocation
  Tt->alloc_nodes ();
  //  allocation of arrays defined on nodes
  Mt->alloc_nodes_arrays ();
  //  number of components in arrays used in outdriver
  //  maximum number of components in array strain/stress on nodes and elements
  Mt->give_maxncompstr(Mm->max_ncompstrn, Mm->max_ncompstre);
  //  maximum number of components in array other on nodes and elements
  Mt->give_maxncompo(Mm->max_nncompo, Mm->max_encompo);

  //  prescribed displacements on elements investigation
  Mt->elemprescdisp ();
  //  reactions on elements investigation
  if (Mp->reactcomp==1){
    Mt->comreac ();
  }

  //  computation of initial values at integration points from initial nodal values
  if (Mb->ico > 0)
    Mb->inicipval();
    
  if (Mp->matmodel == nonlocal){
    //  list of adjacent elements
    Gtm->adjacelem (Out);
    //  list of adjacent integration points was not restored 
    //  from file it is necessary to create it from scratch
    if (restore_adjacip() == 0)
      adjacip ();
    save_adjacip();
    ipvolume ();
  }

  //  allocation of eigenstrains
  //  eigenstrains contain strains generated by coupling terms
  if (Mp->eigstrains!=0){
    Mm->alloceigstrains ();
    Mm->alloceigstresses ();
  }

  if (Mp->pore_press!=0){
    Mm->alloceigstresses ();
  }

  //  initiation of temprstrains
  if (Mp->temperature!=0)
    {
      Mm->alloctempstrains ();//tohle jeste rozmyslet??!!
      Mm->alloceigstresses ();
    }
  
  //Mt->nodedisplacements (); //tohle jeste rozmyslet??!!

  xfclose (in);
  xfclose (inm);
  xfclose (intr);
}

void print_helpc(FILE *out, const char *prgname)
{
  fflush(out);
  fprintf(out, "\nInput file has not been specified\n");
  fprintf(out, " use : %s infile [-kwd=i]\n", prgname);
  fprintf(out, " where :\n *   infile is input file name\n");
  fprintf(out, " *   -kwd={0,1,2,3,4} is optional switch which controls keyword usage in the input file\n");
  fprintf(out, "           0 means no keywords (default value)\n");
  fprintf(out, "           1 means keywords only in the problem description section\n");
  fprintf(out, "           2 means keywords only in the outdriver section\n");
  fprintf(out, "           3 means keywords in the problem description and outdriver sections\n");
  fprintf(out, "           4 means full usage of keywords\n\n");
  abort();
}


long process_argsc(int argc, const char *argv[])
{
  const char *pkwd;
  
  switch (argc)
  {
    case 0:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too few arguments are used\n\n");
      print_helpc(stderr, "pmetr|metr");
      return 1;
    case 1:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too few arguments are used\n\n");
      print_helpc(stderr,argv[0]);
      return 1;
    case 2:
      Cp->kwdsw = nokwd;
      break;
    case 3:
      // processing of optional keyword switch -kwd={0,1,2,3,4}
      pkwd = strstrcis(argv[2], "-kwd=");
      if (pkwd)
      {
        Cp->kwdsw = pkwd_sw(-1);
        sscanf(pkwd+5, "%d", (int *)&Cp->kwdsw);
        if ((Cp->kwdsw < 0) || (Cp->kwdsw > 4))
        {
          fflush(stdout);
          fprintf(stderr, "\n\nError - unknown -kwd value %d is obtained\n\n", Cp->kwdsw);
          print_helpc(stderr,argv[0]);
          return 1;
        }
      }
      else
      {
        fflush(stdout);
        fprintf(stderr, "\n\nError - unknown switch is used\n\n");
        print_helpc(stderr,argv[0]);
        return 1;
      }
      break;
    default:
      fflush(stdout);
      fprintf(stderr, "\n\nError - too many arguments is used\n\n");
      print_helpc(stderr,argv[0]);
      return 1;
  }
  return 0;
}
