#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "probdescc.h"
#include "globalc.h"
#include "gmatrix.h"
#include "intools.h"



probdescc::probdescc (void)
{
  kwdsw = nokwd;

  niilnr=0;  errnr=0.0;
  alpha=0.0;  time=0.0;
  zero = 1.0e-20;

  
  lbb = notdeflbb;

  path = filename = suffix = NULL;
  //  smaller number of integration points
  savemode=1;
  
  // clean matrices
  cleanmatrix = (coupcleanmatrices) 0;

  //  type of nonlinear solver
  fcsolv = (coupsolver) 0;
  
  //  type of residuum
  restype = (residuumtype) 0;
  
  //  solver of system of linear algebraic equations
  ssle = new slesolv();
}

probdescc::~probdescc (void)
{
  delete [] path; delete [] filename; delete [] suffix;
  delete ssle;
}

/**
   function reads basic data about solved problem
   
   @param in - input stream
   
   23.11.2002
*/
void probdescc::read (XFILE *in)
{
  char str_tmp[1001];//auxiliary string
  const char *pkwd;
  
  // problem description
  xfscanf (in," %a",name);
  // line with MEFEL input filename and optional -kwd switch
  xfscanf (in," %a",str_tmp);
  // processing of optional keyword switch -kwd={0,1,2,3,4}
  pkwd = strstrcis(str_tmp, "-kwd=");
  if (pkwd)
  {
    // pickup MEFEL input filename from the beginning of line until the first whitespace occurence
    memset(minfile, 0, 1001*sizeof(*minfile));
    strncpy (minfile, str_tmp, (strchr(str_tmp,' ')-str_tmp)/sizeof(char));
    Mp->kwdsw = pkwd_sw(-1);
    sscanf(pkwd+5, "%d", (int *)&Mp->kwdsw);
    if ((Mp->kwdsw < 0) || (Mp->kwdsw > 4))
    {
      fflush(stdout);
      fprintf(stderr, "\n\nError - unknown -kwd value %d is obtained\n\n", Mp->kwdsw);
      abort();
    }
  }
  else
  {
    strcpy (minfile, str_tmp);
    Mp->kwdsw = nokwd;
  }
  
  // line with TRFEL input filename and optional -kwd switch
  xfscanf (in," %a",str_tmp);
  // processing of optional keyword switch -kwd={0,1,2,3,4}
  pkwd = strstrcis(str_tmp, "-kwd=");
  if (pkwd)
  {
    memset(tinfile, 0, 1001*sizeof(*tinfile));
    // pickup TRFEL input filename from the beginning of line until the first whitespace occurence
    strncpy (tinfile, str_tmp, (strchr(str_tmp,' ')-str_tmp)/sizeof(char));
    Tp->kwdsw = pkwd_sw(-1);
    sscanf(pkwd+5, "%d", (int *)&Tp->kwdsw);
    if ((Tp->kwdsw < 0) || (Tp->kwdsw > 4))
    {
      fflush(stdout);
      fprintf(stderr, "\n\nError - unknown -kwd value %d is obtained\n\n", Mp->kwdsw);
      abort();
    }
  }
  else
  {
    strcpy (tinfile, str_tmp);
    Tp->kwdsw = nokwd;
  }

  //  detail messages
  Mesprc=0;
  xfscanf (in,"%k%ld","mesprc",&Mesprc);
  if (Mesprc==1)  fprintf (stdout,"\n detail information will be printed\n");
  else fprintf (stdout,"\n only important messages will be printed\n");
 
  //  problem type
  xfscanf (in,"%k%m","problemtype",&problemtypec_kwdset,(int*)&tprob);
  //  Ladyzhenskaja-Babuska-Brezzi
  xfscanf (in,"%k%m","LBB",&ladbabuskabrezzi_kwdset,(int*)&lbb);
  //  transported matter
  xfscanf (in,"%k%m","transmatter",&transmatterc_kwdset,(int*)&tmatt);
  //  names of transported media
  xfscanf (in,"%k%m","mednames",&mednamesc_kwdset,(int*)&mednam);
  
  switch (tprob){
  case fully_coupled_mech_trans:{
    
    //  time controller
    //  start time, end time, number of important times
    //  list of important times, time increment function
    timecon.read (in);
    
    //  alpha coefficient
    xfscanf (in,"%lf",&alpha);
    
    //  parameters of Newton-Raphson method
    //  maximum number of iteration in one increment
    //  maximum norm of residual vector
    xfscanf (in,"%d",(int*)&tnlinsol);
    xfscanf (in,"%ld %lf",&niilnr,&errnr);
    
    //  type of nonlinear solver
    xfscanf (in,"%k%m%k%m", "coupledsolver", &coupsolver_kwdset, (int*)&fcsolv, "resid_type", &residuumtype_kwdset, (int*)&restype);
        
    //  type of storage of zero-order and first order matrices
    //  zero-order matrix = stiffness and conductivity matrices
    //  first-order matrix = capacity matrix
    xfscanf (in,"%k%m%k%m","zeroordermat",&storagetype_kwdset,(int*)&tstord0,"firstordermat",&storagetype_kwdset,(int*)&tstord1);
    
    //  data about solver of system of linear equations
    ssle->read (Gtu,in,Mesprc);
    
    break;
  }
  case growing_par_coupl_mech_trans:
  case par_coupl_mech_trans:{
    
    //  time controller
    //  start time, end time, number of important times
    //  list of important times, time increment function
    timecon.read (in);
    
    //  linearc=0
    //  fullnewtonc=1
    //  modnewtonc=2
    xfscanf (in,"%k%m","nonlinsolvertype",&nonlinsolvertypec_kwdset,(int*)&tnlinsol);

    xfscanf (in,"%ld %lf",&niilnr,&errnr);
    
    //  type of coupled solver
    xfscanf (in,"%k%m","cleanmatrices",&coupcleanmatrices_kwdset,(int*)&cleanmatrix);
    
    //  this problem requires storage of actual nodal values on nodes
    Tp->nvs=1;
    //  this problem requires storage of actual nodal values on nodes
    Tp->pnvs=1;
    //  this problem requires storage of initial nodal values on nodes
    Tp->invs=1;
    //  this problem requires storage of time derivatives of nodal values on nodes
    //tdnvs=1;

    break;
  }
  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
    abort ();
  }
  }
}


/**  function prints basic data about solved problem
     JK, 25.9.2001
     
     @param in - output file
*/
void probdescc::print (FILE *out)
{
  //  problem name
  fprintf (out, "%s\n", name);

  //  MEFEL input file name
  fprintf (out,"%s\n",minfile);

  //  TRFEL input file name
  fprintf (out,"%s\n",tinfile);

  //  detail messages
  fprintf (out,"%ld\n", Mesprc);

  //  problem type
  fprintf (out,"%d", tprob);
  
  // LBB condition
  fprintf (out,"%d", lbb);

  //  transported matter
  fprintf (out," %d", tmatt);
  
  //  names of transported media
  fprintf (out, " %d\n", mednam);
     
  switch (tprob){
  case fully_coupled_mech_trans:{
    
    timecon.print (out);

    fprintf (out," %d",tnlinsol);

    fprintf (out, " %lf \n", alpha);

    fprintf (out," %ld %lf\n",niilnr,errnr);
    
    fprintf (out," %d %d\n",fcsolv,restype);

    fprintf (out," %d %d\n",tstord0,tstord1);
    
    ssle->print (out);
    
    fprintf (out,"\n");
    
    break;
  }

  case growing_par_coupl_mech_trans:
  case par_coupl_mech_trans:{
    
    timecon.print (out);
    
    
    fprintf (out," %d",tnlinsol);

    fprintf (out," %ld %lf\n",niilnr,errnr);
    
    fprintf (out," %d\n",fcsolv);

    break;
  }

  default:{
    print_err("unknown problem type is required",__FILE__,__LINE__,__func__);
  }
  }  
}
